from meresco.components import StorageComponent, XmlParseLxml, Lxml2Amara, XmlXPath, XmlPrintLxml, XmlParseAmara, XmlPrintAmara, Amara2Lxml, Venturi, Configuration, FilterField, RenameField, TransformField, RewritePartname, Xml2Fields, XPath2Field, FilterMessages, Rss, RssItem, Fields2XmlTx
from meresco.components.statistics import Statistics, Logger
from meresco.components.statisticsxml import StatisticsXml
from meresco.components.drilldown import Drilldown, SRUDrilldownAdapter, SRUTermDrilldown, DrilldownRequestFieldnameMap
from meresco.components.http import PathFilter, FileServer, PathRename
from meresco.components.http.webrequestserver import WebRequestServer
from meresco.components.lucene import LuceneIndex, unlock, CQL2LuceneQuery, Fields2LuceneDocumentTx
from meresco.components.sru import Sru, SRURecordUpdate

from meresco.framework import Transparant, TransactionScope, TransactionFactory, Observable

from cq2utils import DirectoryWatcher

from os.path import join, isdir
from os import remove, makedirs

from dynamichtml import DynamicHtml

from delfttilburg.normalize import YearNormalize, LastnameNormalize, FirstInitialNormalize
from delfttilburg import RateRecordCookie, RateRecord, MultiLevelDrilldown, ComposeNameFromMods, ModsDateField
from bzv.utils import HandleRequestPrinter
from bzv.openurl import OpenURL

unqualifiedTermFields = [('mods', 1.0), ('meta', 0.1)]

drilldownFieldnames = [
#    'drilldown.mods.name.namePart',
#    'drilldown.mods.name.type',
#    'drilldown.mods.name.role.roleTerm',
    'drilldown.mods.classification',
    'drilldown.mods.genre',
    'drilldown.mods.language.languageTerm',
    'drilldown.mods.relatedItem.titleInfo.title',
    'drilldown.mods.subject.topic',
    'drilldown.meta.repository.id',
    'drilldown.meta.repository.repositoryGroupId',
    'drilldown.norm.date.unparsable',
    'drilldown.norm.date.year',
    'drilldown.norm.date.decennium',
    'drilldown.norm.date.centennium',
    'drilldown.norm.name.oneinitial',
    'drilldown.norm.name.lastname',
    'drilldown.norm.name.unparsable',
]

sortFieldnames = [
    'sort.mods.originInfo.dateIssued',
]
metaFieldnames = [
    'meta.upload.id',
    'meta.record.id',
    'meta.repository.id',
    'meta.repository.set',
    'meta.repository.baseurl',
    'meta.repository.repositoryGroupId',
    'meta.repository.metadataPrefix',
    'meta.repository.collection',
]

DRILLDOWN_PREFIX = 'drilldown.'
SORT_PREFIX = 'sort.'


namespaces = {
    'oai_dc':"http://www.openarchives.org/OAI/2.0/oai_dc/",
    'meta': "http://meresco.com/namespace/harvester/meta",
    'mods': 'http://www.loc.gov/mods/v3'
}

class ServerLog(Observable, Logger):
    def handleRequest(self, Client=None, *args, **kwargs):
        if Client:
            self.log(ip=Client[0])
        self.log(request='request')
        return self.all.handleRequest(Client=Client, *args, **kwargs)

def createUploadHelix(index, storageComponent):
    yearNormalize = YearNormalize(yearRange=(1400,2100))
    firstInitialNormalize = FirstInitialNormalize()
    lastnameNormalize = LastnameNormalize()
    fields2LuceneDocument = (TransactionFactory(lambda tx: Fields2LuceneDocumentTx(tx, untokenized=drilldownFieldnames + sortFieldnames + metaFieldnames)),
        index
    )
    indexingHelix = \
        (Transparant(),
            fields2LuceneDocument,
            (FilterField(lambda name: DRILLDOWN_PREFIX + name in drilldownFieldnames),
                (RenameField(lambda name: DRILLDOWN_PREFIX + name),
                    fields2LuceneDocument
                )
            ),
            (FilterField(lambda name: SORT_PREFIX + name in sortFieldnames),
                (RenameField(lambda name: SORT_PREFIX + name),
                    fields2LuceneDocument
                )
            ),
        )

    normalizeTransactionStoreHelix = \
        (TransactionFactory(lambda tx:Fields2XmlTx(tx, 'extra', namespace="http://meresco.com/namespace/fields/extra")),
            (storageComponent,)
        )

    indexAndStoreModsHelix = \
        (RewritePartname('mods'),
            (XmlPrintLxml(),
                (storageComponent,)
            ),
            (Xml2Fields(),
                indexingHelix,
                (RenameField(lambda name: name.split('.')[0]),
                    indexingHelix
                ),
                (FilterField(lambda name: name == '__id__'),
                    normalizeTransactionStoreHelix
                )
            ),
            (ComposeNameFromMods('norm.name.oneinitial'),
                (TransformField(firstInitialNormalize.normalize),
                    indexingHelix,
                    normalizeTransactionStoreHelix,
                ),
                (TransformField(firstInitialNormalize.unparsable),
                    (RenameField(lambda name: 'norm.name.unparsable'),
                        indexingHelix
                    )
                ),
                (TransformField(lastnameNormalize.normalize),
                    (RenameField(lambda name: 'norm.name.lastname'),
                        indexingHelix,
                        normalizeTransactionStoreHelix,
                    )
                )
            ),
            (XPath2Field([('/mods:mods/mods:name//@type', 'mods.name.type')], namespaceMap={'mods': 'http://www.loc.gov/mods/v3'}),
                indexingHelix
            ),
            (XPath2Field([('/mods:mods/mods:originInfo/mods:dateIssued/text()', 'norm.date.year')], namespaceMap={'mods': 'http://www.loc.gov/mods/v3'}),
                (TransformField(yearNormalize.normalize),
                    indexingHelix,
                    normalizeTransactionStoreHelix,
                    (TransformField(lambda year: year[:len('YYY')]+'0'),
                        (RenameField(lambda name: 'norm.date.decennium'),
                            indexingHelix,
                            normalizeTransactionStoreHelix
                        )
                    ),
                    (TransformField(lambda year: year[:len('YY')]+'00'),
                        (RenameField(lambda name: 'norm.date.centennium'),
                            indexingHelix,
                            normalizeTransactionStoreHelix
                        )
                    )
                ),
                (TransformField(yearNormalize.unparsable),
                    (RenameField(lambda name: 'norm.date.unparsable'),
                        indexingHelix
                    )
                )
            ),
            (ModsDateField(),
                indexingHelix,
            ),
        )

    return \
        (Venturi(
                should=[('metadata', '/document:document/document:part[@name="metadata"]/text()'),
                ('meta', '/document:document/document:part[@name="meta"]/text()'),
                ('header', '/document:document/document:part[@name="header"]/text()'),],
                could=[('rating', '/rating:rating') ],
                namespaceMap={'document':'http://meresco.com/namespace/harvester/document',
                'rating': "http://meresco.com/namespace/record/rating"}
            ),

            (FilterMessages(allowed=['getStream']),
                (storageComponent,)
            ),

            (XmlPrintLxml(),
                (storageComponent,)
            ),

            (XmlXPath(['/oai:metadata/oai_dc:dc']),
                (XmlPrintLxml(),
                    (RewritePartname('dc'),
                        (storageComponent,)
                    )
                ),
                indexingHelix,
                (Xml2Fields(),
                    (RenameField(lambda name: name.split('.')[0]),
                        indexingHelix
                    )
                )
            ),
########### this became redundant after separating the storing and indexing of mods from the storing of didl
#            (XmlXPath(['/oai:metadata/mods:mods'], namespaceMap={'mods': 'http://www.loc.gov/mods/v3'}),
#                indexAndStoreModsHelix
#            ),
            (XmlXPath(['/oai:metadata/didl:DIDL'], namespaceMap={'didl':'urn:mpeg:mpeg21:2002:02-DIDL-NS'}),
                (XmlPrintLxml(),
                    (RewritePartname('didl'),
                        (storageComponent,)
                    )
                )
            ),
########### separated the storing and indexing of mods from the storing of didl
            (XmlXPath(['//mods:mods'], namespaceMap={'mods': 'http://www.loc.gov/mods/v3'}),
                    indexAndStoreModsHelix
            ),

            (XmlXPath(['/meta:meta']),
                (Xml2Fields(),
                    indexingHelix,
                    (FilterField(lambda name: name == 'meta.repository.id'),
                        (RenameField(lambda name: 'meresco.exists'),
                            (TransformField(lambda value: 'true'),
                                indexingHelix
                            )
                        )
                    ),
                    (RenameField(lambda name: 'meta'),
                        indexingHelix
                    )
                ),
            )
        )


def main(reactor, observableHttpServer, sruUpdateServer, host, portNumber, databasePath, configDict):
    storageDirectory = join(databasePath, 'storage')
    indexDirectory = join(databasePath, 'index')

    dynamicPath = configDict['dynamicPath']
    staticPath  = configDict['staticPath']
    unlock(indexDirectory)

    storageComponent = StorageComponent(storageDirectory)
    drilldownComponent = Drilldown(drilldownFieldnames)

    indexHelix = \
            (LuceneIndex(indexDirectory,timer=reactor),
                (drilldownComponent,)
            )

    uploadHelix = createUploadHelix(indexHelix, storageComponent)

    configurationComponent = Configuration(configDict)
    rateRecordList = RateRecordCookie(join(databasePath, 'ratedRecords'))

    statisticsPath = join(databasePath, 'statistics')
    isdir(statisticsPath) or makedirs(statisticsPath)
    statistics = Statistics(statisticsPath, [('ip',), ('clause',)], 60)

    return \
        (Observable(),
            (sruUpdateServer,
                (PathFilter('/update'),
                    (WebRequestServer(),
                        (SRURecordUpdate(),
                            (Amara2Lxml(),
                                (TransactionScope(),
                                    uploadHelix
                                )
                            )
                        )
                    )
                ),
            ),
            (observableHttpServer,
                (statistics,
                    (ServerLog(), ),
                    (PathFilter('/statistics'),
                        (StatisticsXml(statistics), )
                    ),
                    (PathFilter('/rss'),
                        (Rss(   title = configDict.get('applicationName', ''),
                                description = 'Universiteit van Tilburg',
                                link = 'http://' + host,
                                maximumRecords = 15),
                            (CQL2LuceneQuery(unqualifiedTermFields),
                                indexHelix
                            ),
                            (RssItem(
                                    nsMap=namespaces,
                                    title = ('mods', '/mods:mods/mods:titleInfo/mods:title/text()'),
                                    description = ('mods', '/mods:mods/mods:abstract/text()'),
                                    linkTemplate = 'http://' + host + '/recordview/?recordId=%(uploadId)s',
                                    uploadId = ('meta', '/meta:meta/meta:upload/meta:id/text()')),
                                (storageComponent, )
                            ),
                        )
                    ),
                    (PathFilter('/sru'),
                        (HandleRequestPrinter(),),
                        (Sru(host=host, port=portNumber, defaultRecordSchema='mods', defaultRecordPacking='xml'),
                            (CQL2LuceneQuery(unqualifiedTermFields),
                                indexHelix
                            ),
                            (storageComponent,),
                            (SRUDrilldownAdapter(),
                                (SRUTermDrilldown(),
                                    (DrilldownRequestFieldnameMap(
                                        lambda field: DRILLDOWN_PREFIX + field,
                                        lambda field: field[len(DRILLDOWN_PREFIX):]),
                                            (drilldownComponent,)
                                    )
                                )
                            )
                        )
                    ),
                    (PathFilter('/files'),\
                        (PathRename(lambda path: path[len('/files'):]),
                            (FileServer(staticPath),)
                        )
                    ),
                    (PathFilter('/', excluding=['/rss', '/sru', '/update', '/files', '/statistics']),
                        (DynamicHtml([dynamicPath], reactor=reactor, allowedModules=['cqlparser', 'cqlwebutils'], indexPage='/nl/page/search'),
                            (CQL2LuceneQuery(unqualifiedTermFields),
                                indexHelix
                            ),
                            #(OpenURL('http://radix-21.uvt.nl:4080/locator/request'),),
                            (OpenURL('http://orac.uvt.nl:8180/locator-1.0-SNAPSHOT/Locator'),),
                            (storageComponent,),
                            (drilldownComponent,),
                            (RateRecord(1,5),
                                (FilterMessages(allowed=['isAvailable', 'getStream']),
                                    (storageComponent,),
                                ),
                                (TransactionScope(),
                                    uploadHelix
                                )
                            ),
                            (MultiLevelDrilldown({
                                    'date': [
                                        ('drilldown.norm.date.year', 20, False),
                                        ('drilldown.norm.date.decennium', 20, False),
                                        ('drilldown.norm.date.centennium', 10, False)
                                        ],
                                    'author': [
                                        ('drilldown.norm.name.oneinitial', 20, False),
                                        ('drilldown.norm.name.lastname', 10, True)
                                        ]
                                    }
                                ),
                                (drilldownComponent,),
                            ),
                            (configurationComponent,),
                            (rateRecordList, ),
                        )
                    )
                )
            )
        )
